%==========================================================================
%             S O L V E U R   N A V I ER -S T O K E S   2D  
%==========================================================================
% Solveur des équations de Navier-Stokes 2D avec conditions de Dirichlet 
% sur le bord.
%
% Domaine sans ou avec obstacle (non nécessairement simplement connexe).
%
% Conditions limites de Dirichlet : u=g sur les bords du domaine (g peut 
% dépendre du temps).
%
% * Elements finis P2/P1 avec une méthode de caractéristique pour 
%   la nonlinéarité (ordre 2 en temps).
%
% * Maillage du domaine par mesh2d v2.3 (Darren Engwirda)
%
%--------------------------------------------------------------------------
% La définition du problème à résoudre se fait avec les fonctions suivantes
%
%   initDIR.m : Paramètres physiques et numériques (viscosité, pas de
%               temps,...)
%   geomDIR.m : Géométrie et paramètres du maillage
%
%   gclDIR.m  : Conditions limites sur le(s) bord(s) du domaine.
%
%   UPinit.m  : Vitesse et pression initiale
%
%--------------------------------------------------------------------------
% Exemples de problèmes traités (variable 'exemple' dans initDIR.m)
%    
%   Exemple 1: Cavité entrainée sur le bord supérieur d'un domaine 
%   rectangle. Conditions limites indépendantes du temps u=(1,0) sur le
%   bord supérieur du domaine.
%
%   Exemple 2: Allées de Von Karman dans un domaine rectangle avec obstacle
%   (cercle /ellipse) et un écoulement de Poiseuille en entrée/sortie.
%   Conditions limites dépendantes du temps.
%==========================================================================
%
% Attention : Il faut d'abord compiler le fichier 'convect.cpp' (mexfile)
% dans la librairie 'lib'. Pour cela, se mettre dans le répertoire ./lib 
% et lancer une des deux commandes suivantes :
% 
% Pour les plateformes 32-bits :
% >> mex -v -O convect.cpp
%
% Pour les plateformes 64 bits :
% >> mex -v -O -largeArrayDims convect.cpp
%
%==========================================================================

close all;
clear all;

global T exemple

% Path pour mesh2d (maillage)
addpath '../Mesh2d_v23b/' 

% Path pour la librairie lib
addpath 'lib/' 

tfull=tic;
fprintf('===========================================================\n');
fprintf('                 SOLVEUR NAVIER-STOKES 2D                  \n');
fprintf('===========================================================\n');

%--------------------------------------------------------------------------
% Paramètres physiques et numériques
%--------------------------------------------------------------------------
% exemple        : choix du problème à résoudre :
%                  1-> Cavité entrainée; 
%                  2-> Allée de Von Karman
% nu             : viscosité
% dt             : pas de temps
% nbiter         : nb d'iterations en temps
% type_affichage : type d'affichage : 0 -> pas de sortie graphique; 
%                                     1 -> vitesse; 2-> pression
% save_name      : repertoire de sauvegarde pour les resultats
% iterverbose    : mode verbeux dans les iterations

outinit=initDIR();

exemple=outinit.exemple;
nu=outinit.nu;
dt=outinit.dt;
nbiter=outinit.nbiter;
type_affichage=outinit.type_print;
save_name=outinit.save_name;
iterverbose=outinit.iterverbose;

if exemple==1,
    fprintf('                     Cavité entrainée\n');
else
    fprintf('                   Allées de Von Karman\n');
end
fprintf('===========================================================\n');
%--------------------------------------------------------------------------
% Définition de la géométrie et parametres du maillage
%--------------------------------------------------------------------------
% node    : coordonnées [x;y] des noeuds du bord du domaine
% edge    : aretes du bord définies par les numeros des noeuds [n1, n2]
% pas_h   : pas de discretisation du maillage
% hdata   : taille(s) du maillage
% options : options pour le maillage (dhmax,verbose...)

[node,edge,pas_h,hdata,optionmesh]=geomDIR();  
  
%--------------------------------------------------------------------------
% Maillages P1 et P2
%--------------------------------------------------------------------------

outmesh=meshP1P2(node,edge,hdata,optionmesh);
    
v1=outmesh.v1; t1=outmesh.t1; e1=outmesh.e1;
isupp1=outmesh.isupp1; tsupp1=outmesh.tsupp1;
v2=outmesh.v2; t2=outmesh.t2; e2=outmesh.e2;
isupp2=outmesh.isupp2; tsupp2=outmesh.tsupp2;

ib2=outmesh.ib2; ic2=outmesh.ic2;
ibT=outmesh.ibT; icT=outmesh.icT; 

% nv1, nt1, ne1 : nb de noeuds, de triangles et d'aretes du maillage P1
% nv2, nt2, ne2 : nb de noeuds, de triangles et d'aretes du maillage P2
nv1=size(v1,1); nt1=size(t1,1); ne1=size(e1,1);
nv2=size(v2,1); nt2=size(t2,1); ne2=size(e2,1);
        
fprintf('Maillage P1 : nombre de noeuds    (nv1) = %d\n',nv1);
fprintf('              nombre de triangles (nt1) = %d\n',nt1);
fprintf('Maillage P2 : nombre de noeuds    (nv2) = %d\n',nv2);
fprintf('              nombre de triangles (nt2) = %d\n',nt2);

% Affichage du maillage
%patch('faces',t2(:,1:3),'vertices',v2(:,1:2),'facecolor','white','edgecolor','blue');
%axis equal; axis off

%--------------------------------------------------------------------------
% Pas de discretisation en temps
%--------------------------------------------------------------------------

pas=max(sqrt(t2(:,5)));   % taille (réelle) du maillage P2
if isempty(dt),
    dt=pas^(1/3)/3;       % pas de discretisation en temps 
                          % (dt=pas si convection d'ordre 1)
end
T=nbiter*dt;              % temps final de calcul

%--------------------------------------------------------------------------
% Construction des matrices de Stokes   
%--------------------------------------------------------------------------

[A,A1ibT,A2ibT,SS,icT]=matrixStokes(v1,t1,nv2,ibT,icT,nu,dt);

fprintf('Taille de la matrice de Stokes : %d x %d (nnz = %d)\n',size(A), nnz(A));

% Factorisation LU de la matrice A : P*A*Q =L*U

fprintf('Factorisation LU...\n');
tstart=tic;
[LA,UA,Pm,Qm]=lu(A,'vector');
fprintf('Temps de la factorisation LU : %f s\n',toc(tstart));

%--------------------------------------------------------------------------
% Conditions limites pour la vitesse
%--------------------------------------------------------------------------

[ubd,timedep]=gclDIR(v2(ib2,:));

% Traitement des conditions de Dirichlet (second membre)
b1=-A1ibT*ubd;    % pour it=1  (convection ordre 1)

% Si les CL sont independantes du temps, on traite une fois pour toute les
% CL pour it>1 (convection ordre 2)
if ~timedep,
    b2=-A2ibT*ubd;    
end

%--------------------------------------------------------------------------
% Initialisation iteration NS
%--------------------------------------------------------------------------

it=1;
fprintf('-----------------------------------------------------------\n');
fprintf('Iterations en temps                                        \n');
fprintf('-----------------------------------------------------------\n');
fprintf('Nombre d''iterations en temps : %d\n',nbiter);
if iterverbose,
   fprintf('Itération : %d\n',it);
end

% Vitesse et pression initiales
[U1x,U1y,Pr]=UPinit(v1,t1,v2,t2);

% Convection
tstart=tic;
V=convect(length(U1x),full(U1x),full(U1y),full(U1x),full(U1y),dt,...
          v2(:,1:2),t2(:,1:3),isupp2',tsupp2);
U1xm=V(1:nv2);
U1ym=V(nv2+1:end);

if iterverbose,
  fprintf('Temps de calcul de la convection %f s\n',toc(tstart));
end

% Second membre et retrait des points appartenant au bord
ld=[SS*U1xm;SS*U1ym;sparse(nv1+1,1)]/dt;
ld=b1+ld;
ld(ibT)=[];

% Résolution du système linéaire (sol = A\ld)
tstart=tic;
sol=zeros(size(A,1),1);
s1=UA\(LA\ld(Pm));
sol(Qm)=s1;

if iterverbose,
  fprintf('Temps de résolution du système linéaire: %f s\n',toc(tstart));
end

u=zeros(2*nv2+nv1+1,1);
u(icT)=sol;
u(ibT)=ubd;

% Sauvegarde des vecteurs vitesse et pression
U2x=u(1:nv2);
U2y=u((nv2+1):(2*nv2));
Pr=u((2*nv2+1):end-1);

%--------------------------------------------------------------------------
% Affichage solution
%--------------------------------------------------------------------------
if type_affichage~=0,
    plotsol(t1,t2,v1,v2,U2x,U2y,Pr,dt,type_affichage);
end

%--------------------------------------------------------------------------
% Sauvegarde des fichiers pour l'iteration it=1
%--------------------------------------------------------------------------
[success,message]=mkdir(save_name);

nom_fich=strcat(save_name,'/','NS_numdata.dat');
save(nom_fich,'nbiter','pas_h','dt','T','nu','-ASCII');
nom_fich=strcat(save_name,'/','NS_v2.dat');
save(nom_fich,'v2');
nom_fich=strcat(save_name,'/','NS_t2.dat');
save(nom_fich,'t2')
nom_fich=strcat(save_name,'/','NS_v1.dat');
save(nom_fich,'v1')
nom_fich=strcat(save_name,'/','NS_t1.dat');
save(nom_fich,'t1')
nom_fich=strcat(save_name,'/','NS_Ux_0001.dat');
save(nom_fich,'U2x')
nom_fich=strcat(save_name,'/','NS_Uy_0001.dat');
save(nom_fich,'U2y')
nom_fich=strcat(save_name,'/','NS_Pr_0001.dat');
save(nom_fich,'Pr');

%==========================================================================
% Itérations Navier-Stokes
%==========================================================================
if ~iterverbose,
    fprintf('Progression des itérations (1...%d)=1',min(20,nbiter));
    htt=(nbiter-1)/19;
    jtt=1;
end

for it=2:nbiter
    
    if iterverbose,
      fprintf('Itération : %d\n',it);
    elseif floor(1+jtt*htt)==it || nbiter<20
            jtt=jtt+1; fprintf('%d',jtt);            
    end
    
    % Mise à jour des conditions limites pour la vitesse dans le cas où 
    % elles dépendent du temps.
    if timedep,
        [ubd,~]=gclDIR(v2(ib2,:),it*dt);
        b2=-A2ibT*ubd;  
    end
    
    % Convection (mexfile)
    tstart=tic;
    Vx=2*U2x-U1x;
    Vy=2*U2y-U1y;
    V=convect(length(U1x),full(U1x),full(U1y),full(Vx),full(Vy),2*dt,...
              v2(:,1:2),t2(:,1:3),isupp2',tsupp2);
    U1xm=V(1:nv2);
    U1ym=V(nv2+1:end);
        
    V=convect(length(U2x),full(U2x),full(U2y),full(Vx),full(Vy),dt,...
              v2(:,1:2),t2(:,1:3),isupp2',tsupp2);
    U2xm=V(1:nv2);
    U2ym=V(nv2+1:end);
    
    if iterverbose,
      fprintf('Temps de calcul de la convection (mexfile): %f s\n',toc(tstart));
    end
    
    % Construction du second membre et retrait des points du bords
    ld=[SS*(2*U2xm-0.5*U1xm);SS*(2*U2ym-0.5*U1ym);sparse(nv1+1,1)]/dt;
    ld=b2+ld;
    ld(ibT)=[];
     
    % Résolution du système linéaire (sol=A\ld)
    tstart=tic;
    s1=UA\(LA\ld(Pm));
    sol(Qm)=s1;
    
    if iterverbose,
      fprintf('Temps de résolution du système linéaire: %f s\n',toc(tstart));
    end
    
    u=zeros(2*nv2+nv1+1,1);
    u(icT)=sol;
    u(ibT)=ubd;

    % Mise à jour de la vitesse et de la pression
    U1x=U2x;
    U1y=U2y;
    U2x=u(1:nv2);
    U2y=u((nv2+1):(2*nv2));
    Pr=u((2*nv2+1):end-1);
     
    %----------------------------------------------------------------------
    % Affichage solution et sauvegarde fichiers
    %----------------------------------------------------------------------
    if type_affichage~=0,
        plotsol(t1,t2,v1,v2,U2x,U2y,Pr,it*dt,type_affichage);
    end
    
    nom_fich=sprintf('%s/NS_Ux_%.4d.dat',save_name,it);
    save(nom_fich,'U2x')
    nom_fich=sprintf('%s/NS_Uy_%.4d.dat',save_name,it);
    save(nom_fich,'U2y')
    nom_fich=sprintf('%s/NS_Pr_%.4d.dat',save_name,it);
    save(nom_fich,'Pr')
    
end

fprintf('\n-----------------------------------------------------------\n');
fprintf('Temps total de résolution : %f s\n',toc(tfull));
