%==========================================================================
%             S O L V E U R   N A V I ER -S T O K E S   2D  
%
%                Version avec interface graphique (GUI)
%
%==========================================================================
% Solveur des équations de Navier-Stokes 2D avec conditions de Dirichlet 
% sur le bord.
%
% Domaine sans ou avec obstacle(s) (ellipse).
%
% Conditions limites de Dirichlet : u=g(t) sur le bord exterieur et u=0 sur
% les bords interieurs
%
% * Elements finis P2/P1 avec une méthode de caractéristique pour 
%   la nonlinéarité (ordre 2 en temps).
%
% * Maillage du domaine par mesh2d v2.3 (Darren Engwirda)
%
%--------------------------------------------------------------------------
% Problèmes proposés comme exemples dans l'interface graphique
%    
%   Exemple 1: Cavité entrainée sur le bord supérieur d'un domaine 
%   rectangle. Conditions limites indépendantes du temps u=(1,0) sur le
%   bord supérieur du domaine.
%
%   Exemple 2: Allées de Von Karman dans un domaine rectangle avec obstacle
%   (cercle /ellipse) et un écoulement de Poiseuille en entrée/sortie.
%   Conditions limites dépendantes du temps.
%==========================================================================
%
% Attention : Il faut d'abord compiler le fichier 'convect.cpp' (mexfile)
% dans la librairie 'lib'. Pour cela, se mettre dans le répertoire ./lib 
% et lancer une des deux commandes suivantes :
% 
% Pour les plateformes 32-bits :
% >> mex -v -O convect.cpp
%
% Pour les plateformes 64 bits :
% >> mex -v -O -largeArrayDims convect.cpp
%
%==========================================================================

close all;
clear all;
 
% Path pour mesh2d (maillage)
addpath '../Mesh2d_v23b/' 

% Path pour la librairie Eléments Finis lib
addpath 'lib/' 

% Path pour la librairie des menus libGUI
addpath 'libGUI/' 

fprintf('===========================================================\n');
fprintf('                 Solveur Navier-Stokes 2D                  \n');
fprintf('===========================================================\n');

ChoixPb = menu_principal;

% Geometrie
[L, H, nbobst] = menu_geom(ChoixPb);
if nbobst~=0,
    [pxy,lgxy,angle,pas_h,dhmax] = menu_obst(nbobst);
else
    [pas_h,dhmax]=menu_mesh;
end

% Conditions limites 
[gcl,timedep]=menu_cl(ChoixPb);

%--------------------------------------------------------------------------
% Construction des bords du domaine
%--------------------------------------------------------------------------
% Bord exterieur du domaine (rectangle) 
nbbord=4;
node = [0 0; L 0; L H; 0 H];
edge = [1 2; 2 3; 3 4; 4 1];
       
for k=1:nbobst
    [node,edge] = obstacle(node,edge,pas_h,pxy(k,:),lgxy(k,:),angle(k));
end

%--------------------------------------------------------------------------
% Maillages P1 et P2
%--------------------------------------------------------------------------
hdata = [];
hdata.hmax  = pas_h;   

if isempty(dhmax),
    optionmesh=[];
else
    optionmesh.dhmax = dhmax;
end
optionmesh.output=false;  % pas de sortie maillage

outmesh=meshP1P2(node,edge,hdata,optionmesh);
    
v1=outmesh.v1; t1=outmesh.t1; e1=outmesh.e1;
isupp1=outmesh.isupp1; tsupp1=outmesh.tsupp1;
v2=outmesh.v2; t2=outmesh.t2; e2=outmesh.e2;
isupp2=outmesh.isupp2; tsupp2=outmesh.tsupp2;

ib2=outmesh.ib2; ic2=outmesh.ic2;
ibT=outmesh.ibT; icT=outmesh.icT; 

% Affichage du maillage P1
h1=figure('Name','Mesh','NumberTitle','off','Position',[35 405 560 420]);
set(h1,'Toolbar','none','MenuBar','none');
patch('faces',t1(:,1:3),'vertices',v1(:,1:2),'facecolor','white',...
      'edgecolor','blue');
axis equal; axis off

% nv1, nt1, ne1 : nb de noeuds, de triangles et d'aretes du maillage P1
% nv2, nt2, ne2 : nb de noeuds, de triangles et d'aretes du maillage P2
nv1=size(v1,1); nt1=size(t1,1); ne1=size(e1,1);
nv2=size(v2,1); nt2=size(t2,1); ne2=size(e2,1);

fprintf('Maillage P1 : nombre de noeuds    (nv1) = %d\n',nv1);
fprintf('              nombre de triangles (nt1) = %d\n',nt1);
fprintf('Maillage P2 : nombre de noeuds    (nv2) = %d\n',nv2);
fprintf('              nombre de triangles (nt2) = %d\n',nt2);

%--------------------------------------------------------------------------
% Pas de discretisation en temps
%--------------------------------------------------------------------------

pas=max(sqrt(t2(:,5))); % Taille (réelle) du maillage P2
dt=pas^(1/3)/3;         % Pour la convection d'ordre 2 en temps, dt doit 
                        % etre plus petit que Cste*h^(1/3) 

%--------------------------------------------------------------------------
% Paramètres physiques et numériques
%--------------------------------------------------------------------------

[nu,nbiter,dt,save_name,type_affichage]=menu_param(dt);

%--------------------------------------------------------------------------
% Construction des matrices de Stokes   
%--------------------------------------------------------------------------

[A,A1ibT,A2ibT,SS,icT] = matrixStokes(v1,t1,nv2,ibT,icT,nu,dt);

% Factorisation LU de la matrice A : P*A*Q =L*U

fprintf('Factorisation LU...\n');
tstart=tic;
[LA,UA,Pm,Qm]=lu(A,'vector');
fprintf('Temps de la factorisation LU : %f s\n',toc(tstart));

tfull=tic;
%--------------------------------------------------------------------------
% Conditions limites pour la vitesse
%--------------------------------------------------------------------------

t=0;             % temps initial
T = nbiter*dt;   % temps final de calcul

% Coordonnées des noeuds du bord
vbd=v2(ib2,:);

% Traitement spécial pour la cavité entrainée sur le bord superieur :
% On choisit les bords auquels sontt rattachés les 4 coins du domaine.
for i=1:nbbord
    indcorn(i)=find(vbd(:,1)==node(i,1) & vbd(:,2)==node(i,2));
end
vbd(indcorn(3),3)=3;
vbd(indcorn(4),3)=3;

ubdx=zeros(length(ib2),1); ubdy=ubdx;
indbd=cell(nbbord,1);
for i=1:nbbord

    ind=find(vbd(:,3)==i);
    x=vbd(ind,1);
    y=vbd(ind,2);
    ubdx(ind) = eval(gcl{i,1}); 
    ubdy(ind) = eval(gcl{i,2}); 
    indbd{i}=ind;
    
end

ubd=[ubdx; ubdy];

% Traitement des conditions de Dirichlet (second membre)
b1=-A1ibT*ubd;    % pour it=1  (convection ordre 1)

% Si les CL sont independantes du temps, on traite une fois pour toute les
% CL pour it>1 (convection ordre 2)
if ~timedep,
    b2=-A2ibT*ubd;    
end

%--------------------------------------
% Initialisation iteration NS
%--------------------------------------
it=1;
fprintf('Iterations en temps                                        \n');
fprintf('-----------------------------------------------------------\n');
fprintf('Nombre d''iterations en temps : %d\n',nbiter);

% Vitesse et pression initiales
[U1x,U1y,Pr]=UPinit(v1,t1,v2,t2);

% Convection
tstart=tic;
V=convect(length(U1x),full(U1x),full(U1y),full(U1x),full(U1y),dt,...
          v2(:,1:2),t2(:,1:3),isupp2',tsupp2);
U1xm=V(1:nv2);
U1ym=V(nv2+1:end);

% Second membre et retrait des points appartenant au bord
ld=[SS*U1xm;SS*U1ym;sparse(nv1+1,1)]/dt;
ld=b1+ld;
ld(ibT)=[];

% Résolution du système linéaire (sol = A\ld)
tstart=tic;
sol=zeros(size(A,1),1);
s1=UA\(LA\ld(Pm));
sol(Qm)=s1;

u=zeros(2*nv2+nv1+1,1);
u(icT)=sol;
u(ibT)=ubd;

% Sauvegarde des vecteurs vitesse et pression
U2x=u(1:nv2);
U2y=u((nv2+1):(2*nv2));
Pr=u((2*nv2+1):end-1);

%--------------------------------------------------------------------------
% Affichage solution
%--------------------------------------------------------------------------
if type_affichage~=0,
    plotsol(t1,t2,v1,v2,U2x,U2y,Pr,dt,type_affichage);
end

%--------------------------------------------------------------------------
% Sauvegarde des fichiers pour l'iteration it=1
%--------------------------------------------------------------------------
[success,message]=mkdir(save_name);

nom_fich=strcat(save_name,'/','NS_numdata.dat');
save(nom_fich,'nbiter','pas_h','dt','T','nu','-ASCII');
nom_fich=strcat(save_name,'/','NS_v2.dat');
save(nom_fich,'v2');
nom_fich=strcat(save_name,'/','NS_t2.dat');
save(nom_fich,'t2')
nom_fich=strcat(save_name,'/','NS_v1.dat');
save(nom_fich,'v1')
nom_fich=strcat(save_name,'/','NS_t1.dat');
save(nom_fich,'t1')
nom_fich=strcat(save_name,'/','NS_Ux_0001.dat');
save(nom_fich,'U2x')
nom_fich=strcat(save_name,'/','NS_Uy_0001.dat');
save(nom_fich,'U2y')
nom_fich=strcat(save_name,'/','NS_Pr_0001.dat');
save(nom_fich,'Pr');

%==========================================================================
% Itérations Navier-Stokes
%==========================================================================

    fprintf('Progression des itérations (1...%d)=1',min(20,nbiter));
    htt=(nbiter-1)/19;
    jtt=1;
        
for it=2:nbiter
    
    t=it*dt;
    
    if floor(1+jtt*htt)==it || nbiter<20
            jtt=jtt+1; fprintf('%d',jtt);            
    end
    
    % Mise à jour des conditions limites pour la vitesse dans le cas où
    % elles dépendent du temps.
    if timedep,
        
        ubdx=zeros(length(ib2),1); ubdy=ubdx;
        for i=1:4
            ind=indbd{i};
            x=vbd(ind,1); y=vbd(ind,2); 
            ubdx(ind) = eval(gcl{i,1}); 
            ubdy(ind) = eval(gcl{i,2}); 
        end

        ubd=[ubdx; ubdy];
        b2=-A2ibT*ubd;  

    end
    
    % Convection (ordre 2 en temps)
   
    Vx=2*U2x-U1x;
    Vy=2*U2y-U1y;
    V=convect(length(U1x),full(U1x),full(U1y),full(Vx),full(Vy),2*dt,...
              v2(:,1:2),t2(:,1:3),isupp2',tsupp2);
    U1xm=V(1:nv2);
    U1ym=V(nv2+1:end);

    V=convect(length(U2x),full(U2x),full(U2y),full(Vx),full(Vy),dt,...
              v2(:,1:2),t2(:,1:3),isupp2',tsupp2);
    U2xm=V(1:nv2);
    U2ym=V(nv2+1:end);
    
    % Construction du second membre et retrait des points du bords
    ld=[SS*(2*U2xm-0.5*U1xm);SS*(2*U2ym-0.5*U1ym);sparse(nv1+1,1)]/dt;
    ld=b2+ld;
    ld(ibT)=[];
     
    % Résolution du système linéaire (sol=A\ld)
    s1=UA\(LA\ld(Pm));
    sol(Qm)=s1;
        
    u=zeros(2*nv2+nv1+1,1);
    u(icT)=sol;
    u(ibT)=ubd;
    
    % Mise à jour de la vitesse et de la pression
    U1x=U2x;
    U1y=U2y;
    U2x=u(1:nv2);
    U2y=u((nv2+1):(2*nv2));
    Pr=u((2*nv2+1):end-1);
    
    %----------------------------------------------------------------------
    % Affichage solution et sauvegarde fichiers
    %----------------------------------------------------------------------
    if type_affichage~=0,
        plotsol(t1,t2,v1,v2,U2x,U2y,Pr,t,type_affichage);
    end  
        
    % Sauvegarde fichiers
    nom_fich=sprintf('%s/NS_Ux_%.4d.dat',save_name,it);
    save(nom_fich,'U2x')
    nom_fich=sprintf('%s/NS_Uy_%.4d.dat',save_name,it);
    save(nom_fich,'U2y')
    nom_fich=sprintf('%s/NS_Pr_%.4d.dat',save_name,it);
    save(nom_fich,'Pr')
    
end

fprintf('\n-----------------------------------------------------------\n');
fprintf('Temps de résolution : %f s\n',toc(tfull));
