function [ubd,timedep]=gclDIR(vbd,tps)
%--------------------------------------------------------------------------
% Condition de Dirichlet pour la vitesse u=g sur le bord du domaine
%--------------------------------------------------------------------------
% vbd     : coordonnées des noeuds du bord (nbdx2)
% tps     : temps; uniquement pour les CL dependantes du temps (exemple 2)
%
% ubd     : vitesse sur le bord
% timedep : booleéen indiquant si les CL dépendent ou non du temps
%
%
% Deux exemples sont proposés :
%
% Exemple 1: Condition limite independante du temps.
%            Cavite entrainee sur le bord superieur d'un domaine rectangle.
%
% Exemple 2: Condition limite dependante du temps.
%            Allée de Von Karman dans un domaine rectangle avec obstacle
%            (cercle /ellipse) et un écoulement de Poiseuille en 
%            entrée/sortie.
%--------------------------------------------------------------------------

global exemple

if nargin==1, tps=0; end;

switch exemple
    
    case 1, 
    %----------------------------------------------------------------------
    % Exemple 1: Cavite entrainee sur le bord superieur d'un domaine 
    % rectangle [0,L]x[0,H]. 
    %  
    % u=(1,0) sur le bord (3) et u=0 ailleurs.
    %
    %                    3 
    %               ------------
    %              |            |
    %            4 |            | 2
    %              |            |
    %               ------------
    %                    1
    %----------------------------------------------------------------------
        
        timedep=false;
        
        % indices des noeuds du bord superieur {y=H}
        ind = find(vbd(:,2)==max(vbd(:,2)));
        x=vbd(ind,1); y=vbd(ind,2);
        z1 = zeros(size(vbd,1),1);
        z2 = z1;
        
        choix=1;
        
        switch choix
         
            case 1, % Cavite entrainee g=(1,0)
                z1(ind)=1;
         
            case 2, % Cavite entrainee "regularisee"
                z1(ind)=1;
                ind1 = find(x<=0.2);
                ind2 = find(x>=0.8);
                z1(ind(ind1)) = 10/0.2*x(ind1);
                z1(ind(ind2)) = 10/0.2*(1-x(ind2));
         
            case 3, % autre choix
                z1 = x.^3 .* y.^2 +1/3;
                z2 = -x.^2 .* y.^3 +2/3;
        
        end
        
        ubd=[z1;z2];
        
    case 2,
    %----------------------------------------------------------------------
    % Exemple 2: Allée de Von Karman dans un domaine rectangle [0,L]x[0,H]
    % avec obstacle (cercle/ellipse) et un écoulement de Poiseuille en 
    % entrée/sortie.
    %                      3 
    %               ----------------
    %              |    __          |
    %              |   /  \         |
    %            4 |  |    |        | 2
    %              |   \__/         |
    %              |                |   
    %               ----------------
    %                      1
    %----------------------------------------------------------------------
        global T
        
        timedep=true;
        
        % indices des noeuds du bord superieur {y=H}
        ind = find(vbd(:,1)==0 | vbd(:,1)==max(vbd(:,1)));
        y=vbd(ind,2);
        z1 = zeros(size(vbd,1),1);
        z2 = z1;

        H=max(vbd(:,2));
        z1(ind)=sin(pi*tps/T)*6*y.*(H-y)/(H^2);
        ubd=[z1;z2];
    
end

end
